#ifndef MPO_FILE_FINDER_H
#define MPO_FILE_FINDER_H

#include "mpo_deleter.h"
#include "mpo_dll.h"

#ifdef WIN32
#include <winsock2.h>
#include <windows.h>
#else
#include <sys/types.h>
#include <dirent.h>
#endif // WIN32

#include <string>
using namespace std;

class IMpoFindResult
{
public:
	virtual wstring GetName() const = 0;

	virtual bool IsDirectory() const = 0;
};

typedef boost::shared_ptr<IMpoFindResult> MpoFindResultSPtr;

class EXPORT_ME MpoFindResult : public IMpoFindResult, public MpoDeleter
{
public:
	static MpoFindResultSPtr GetInstance(const wstring &wstrName, bool bIsDirectory);

	wstring GetName() const;

	bool IsDirectory() const;
private:
	MpoFindResult();

	void DeleteInstance();

	wstring m_wstrName;
	bool m_bIsDirectory;
};

class IMpoFileFinder
{
public:
	virtual MpoFindResultSPtr StartFileFind(const wstring &wstrDirName) = 0;
	
	virtual MpoFindResultSPtr NextFileFind() = 0;
};

typedef boost::shared_ptr<IMpoFileFinder> MpoFileFinderSPtr;

class EXPORT_ME MpoFileFinder : public IMpoFileFinder, public MpoDeleter
{
public:
	static MpoFileFinderSPtr GetInstance();

	MpoFindResultSPtr StartFileFind(const wstring &wstrDirName);
	
	MpoFindResultSPtr NextFileFind();

private:
	MpoFileFinder();
	~MpoFileFinder();

	void DeleteInstance();

	void Shutdown();

#ifdef WIN32
	HANDLE m_FindHandle;
	WIN32_FIND_DATAW m_FindData;
#else
	DIR *m_dir;
	string m_strDirName;	// to strip off directory prefix when returning results
#endif // WIN32

	bool m_bFindStarted;
};

#endif // MPO_FILE_FINDER_H
