#ifndef MPO_LOG_H
#define MPO_LOG_H

#include "mpo_dll.h"
#include "mpo_deleter.h"
#include "mpo_thread.h"
#include <string>
using namespace std;

#define MPO_LOG(pLog, s) pLog->Log(s, __FILE__, __LINE__)
#define MPO_LOGERROR(pLog, s, err) pLog->LogError(s, err, __FILE__, __LINE__)

class EXPORT_ME ThreadSafeLog;

typedef boost::shared_ptr<ThreadSafeLog> ThreadSafeLogSPtr;

class ThreadSafeLog : public MpoDeleter
{
public:
	static ThreadSafeLogSPtr GetInstance(const wstring &wstrLogFileName);

	void Log(const string &strMsg, const string &strFile = "", unsigned int uLine = 0);

	void LogError(const string &strMsg, int iErrCode, const string &strFile = "", unsigned int uLine = 0);

	void LogBuffer(const void *pBuf, size_t stBufSizeBytes);

private:
	ThreadSafeLog();
	~ThreadSafeLog();

	void DeleteInstance();

	bool Init();

	void Shutdown();

	///

	MpoThreadMutexSPtr m_MutexSPtr;
	MpoThreadMutex *m_pMutex;
	bool m_bMutexCreated;

	string m_strLogFileName;

	// Just in case we couldn't open the log file, we buffer what we were gonna write for later
	// (I'm not sure why this happens sometimes)
	string m_strBuffered;

};

#endif // MPO_LOG_H
