/*
 * mpo_xml.h
 *
 * Copyright (C) 2005 Matthew P. Ownby
 *
 * This file is part of MPOLIB, a multi-purpose library
 *
 * MPOLIB is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MPOLIB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// MPO's NOTE:
//  I may wish to use MPOLIB in a proprietary product some day.  Therefore,
//   the only way I can accept other people's changes to my code is if they
//   give me full ownership of those changes.

#ifndef MPO_XML_H
#define MPO_XML_H

#include "mpo_dll.h"
#include <vector>
#include <string>

using namespace std;

EXPORT_ME void xml_el_start(void *data, const char *el, const char **attr);
EXPORT_ME void xml_el_end(void *data, const char *el);
EXPORT_ME void xml_char_data(void *data, const char *msg, int len);

/////////////////

struct xml_attribute
{
	string name;	// name of attribute
	string val;	// value of attribute
};
	
struct xml_element
{
	string name;	// name of element
	vector <xml_attribute> attributes;	// any attributes the element has
	string text;	// any text inside the element
	vector <xml_element> children;	// any sub-elements beneath this element
};

class EXPORT_ME mpo_xml
{
public:
	mpo_xml();
	bool parse(const unsigned char *buf, unsigned int size, string &err_msg);
	const struct xml_element *get_parsed_element();

	// outputs XML in a string (which can be saved to a file)
	string dump_xml(const xml_element *x, int depth = 0);
	
	// searches through all children for an element named 'strName'
	// Returns a pointer to the element if it's found, or NULL if not found.
	const struct xml_element *find_child(const xml_element *pParent, const string &strName, bool bMatchCase = true);

	// scans a string for reserved characters and replaces them with 'entity references'
	// 'bAttribute' indicates whether the string is to be escaped to be used as the value of an xml_attribute
	static string escape(const string &strInp, bool bAttribute);

	void start(const char *el, const char **attr);
	void end(const char *el);
	void text(const char *msg, int len);
private:
	void init();
	xml_element *get_active_child();

	xml_element m_head;
	bool m_got_head;	// whether the head has anything in it or not
	int m_depth;	// 0 = head, 1 = last element of 1st set of children, etc...
	string m_char_data;
};

#endif // MPO_XML_H

